<?php

namespace Teknisa\ACL\Repositories;

use Doctrine\DBAL\Connection;
use Teknisa\ACL\Util\InstanceProvider;

class AccessProfile {

    const QUERY_MAX_NRPERFILACESSO = "SELECT MAX(NRPERFILACESSO) as MAXNRPERFILACESSO FROM PERFILACESSO";

    /** @var Connection */
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function getNrorg() {
        return InstanceProvider::getEnvironment()->getCurrentOrganizationId();
    }

    public function getMaxNrPerfilAcesso () {
        return $this->connection->fetchAll(self::QUERY_MAX_NRPERFILACESSO);
    }
    
    public function findAccessControllUtilization($nrorg){
        try {
            $query = "SELECT IDUTCTRLACESWEB
                        FROM PARAMGERAL
                       WHERE NRORG = ".$nrorg;
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function validateActivation($nrorg){
        $query = "SELECT COUNT(*) AS TOTAL
                    FROM PERFACESSPARC PEPA
                    JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG
                   WHERE PEPA.IDATIVO = 'S'
                     AND PERF.IDATIVO = 'S'
                     AND PERF.IDPERFILACESSOADM = 'S'
                     AND PERF.NRORG = " . $nrorg;
        try {
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function changeAccessProfile($nrorg, $idutctrlacesweb){
        try {
            $query = "UPDATE PARAMGERAL SET IDUTCTRLACESWEB = '".$idutctrlacesweb."' WHERE NRORG = ".$nrorg;
            $this->connection->executeQuery($query);
            return $idutctrlacesweb;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function changeAccessControl($nrorg, $idutlperfilfilial){
        try {
            $query = "UPDATE PARAMGERAL SET IDUTLPERFILFILIAL = '".$idutlperfilfilial."' WHERE NRORG = ".$nrorg;
            $this->connection->executeQuery($query);
            return $idutlperfilfilial;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function insertAccessProfile($nrorg, $idutctrlacesweb, $cdpargeral){
        $insert = "INSERT INTO PARAMGERAL
                              (CDPARGERAL, IDUTCTRLACESWEB, NRORG)
                       VALUES (:CDPARGERAL, :IDUTCTRLACESWEB, :NRORG)";

        $params = array(
            "CDPARGERAL"    =>$cdpargeral,
            "IDUTCTRLACESWEB"  => $idutctrlacesweb,
            "NRORG"         => $nrorg
        );

        try
        {
            $std = $this->connection->prepare($insert);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function findPerfAcessControl($nrorg, $nrAccessProfile){
        try {
            $nrAccessProfile = ($nrAccessProfile == '' ? null : $nrAccessProfile);
            $query = "SELECT PC.NRPERFILACESSO, PA.NMPERFILACESSO
                        FROM PERFACESSCONTROL PC
                        JOIN PERFILACESSO PA ON
                            PA.NRPERFILACESSO = PC.NRPERFILACESSO
                       WHERE PC.NRORG = ".$nrorg;
            if($nrAccessProfile != null){
                $query .= " AND PC.NRPERFACESSCONT = ".$nrAccessProfile;
            }
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    } 
    
    public function findPerfAccessByControlProfile($nrorg, $nrAccessProfile){
        try {
            $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, {$nrAccessProfile} AS NRPERFACESSCONT
                        FROM PERFILACESSO 
                       WHERE NRORG = {$nrorg} 
                            AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSCONTROL WHERE NRPERFACESSCONT = {$nrAccessProfile})
                            ";
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function saveAccessProfileByControlProfile($accessProfile)
    {
        $query = "INSERT INTO PERFACESSCONTROL
                              (NRPERFACESSCONT, NRPERFILACESSO,    NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES ( ".$accessProfile['NRPERFACESSCONT'].", ".$accessProfile['NRPERFILACESSO'].",   ".$accessProfile['NRORG'].", SYSDATE, ".$accessProfile['NRORG'].", ".$accessProfile['CDOPERINCLUSAO'].")";
                               
        try {
            $std = $this->connection->prepare($query);
            return $std->execute(array());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteControlProfile($nrAccessProfile, $nrControlAccess, $organizationId)
    {
        $query = "DELETE 
                    FROM PERFACESSCONTROL
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRPERFACESSCONT = :NRPERFACESSCONT
                     AND NRORG      = :NRORG";
        
        $params = array(
            ':NRPERFILACESSO' => $nrAccessProfile,
            ':NRPERFACESSCONT'   => $nrControlAccess,
            ':NRORG'      => $organizationId
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteAssociatedPerfAcessFili($profilesToDelete, $params)
    {
        $queryPerfAcessFili = "DELETE 
                    FROM PERFACESSFILI
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRORG      = :NRORG";
                     
        $queryPerfAcessControl = "DELETE 
                    FROM PERFACESSCONTROL
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRORG      = :NRORG";
                     
        
        $params = array(
            ':NRPERFILACESSO' => $profilesToDelete[0],
            ':NRORG'    => $params['NRORG']
        );
        
        try {
            $std = $this->connection->prepare($queryPerfAcessFili);
            $std->execute($params);
            $std = $this->connection->prepare($queryPerfAcessControl);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findValAntigoPerfAcess($partnerAccessProfileNumber)
    {

        $query = "SELECT *FROM PERFILACESSO
                  WHERE NRPERFILACESSO = :NRPERFILACESSO
                  AND NRORG = :NRORG";

        $params = array (
            "NRPERFILACESSO" => $partnerAccessProfileNumber,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function findAccessProfileOld($nrorg)
    {

        $query = "SELECT *FROM PARAMGERAL
                  WHERE NRORG = :NRORG";

        $params = array (
            "NRORG" => $nrorg
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function findRules($nrorg, $profile)
    {

        $query = "SELECT *FROM ZHU_ACCESS_CONTROL_RULE
                  WHERE ORGANIZATION_ID = :ORGANIZATION_ID
                  AND GROUP_ID = :GROUP_ID
                  AND NRORG = :NRORG";

        $params = array (
            "ORGANIZATION_ID" => $nrorg,
            "GROUP_ID" =>  $profile,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    } 

    public function findElement($uniqueId, $projectId)
    {

        $query = "SELECT *FROM ZHU_ACCESS_CONTROL_ELEMENT
                  WHERE UNIQUE_ID = :UNIQUE_ID
                  AND PROJECT_ID = :PROJECT_ID
                  AND NRORG = :NRORG";

        $params = array (
            "UNIQUE_ID" => $uniqueId,
            "PROJECT_ID" =>  $projectId,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    } 

    public function findRuleElement($organizationId, $groupId, $elementId)
    {

        $query = "SELECT *FROM ZHU_ACCESS_CONTROL_RULE
                  WHERE ORGANIZATION_ID = :ORGANIZATION_ID
                  AND GROUP_ID = :GROUP_ID
                  AND ACCESS_CONTROL_ELEMENT_ID = :ACCESS_CONTROL_ELEMENT_ID
                  AND NRORG = :NRORG";

        $params = array (
            "ORGANIZATION_ID" => $organizationId,
            "GROUP_ID" =>  $groupId,
            "ACCESS_CONTROL_ELEMENT_ID" =>  $elementId,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    } 

    public function findElementRule($id)
    {

        $query = "SELECT *FROM ZHU_ACCESS_CONTROL_RULE
                  WHERE ID = :ID
                  AND NRORG = :NRORG";

        $params = array (
            "ID" => $id,
            "NRORG"         => $this->getNrorg(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    } 
}